/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * --------------------------------------------------------------------------
 * $Id: AbsJmxServiceImpl.java,v 1.14 2004/05/11 08:03:56 danesa Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.jmx;

import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.remote.JMXServiceURL;
import javax.naming.Context;
import javax.naming.NamingException;

import org.objectweb.carol.util.configuration.CarolConfiguration;
import org.objectweb.jonas.common.JProp;
import org.objectweb.jonas.common.Log;
import org.objectweb.jonas.service.AbsServiceImpl;
import org.objectweb.jonas.service.ServiceException;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * JMX Service implementation. This singleton class must exist in each jonas
 * server that is to be administered via JMX. Its main role is to initialize the 
 * service (the singleton object). 
 * @author Guillaume Riviere
 * @author Michel Bruno
 * @author Adriana Danes
 */

public abstract class AbsJmxServiceImpl extends AbsServiceImpl implements JmxService {

    /**
     * Trace logger
     */
    private static Logger logger = null;

    /**
     * @return Logger logger object
     */
    protected static Logger getLogger() {
        return logger;
    }

    /**
     * the name of the JOnAS server that is administerd by the MBeanServer
     */
    private String jonasName = null;

    /**
     * the name of the management domain
     */
    private String domainName = null;

    /**
     * @return The name of the management domain
     */
    public String getDomainName() {
        return this.domainName;
    }


    /**
     * JMX server (MBean server) instance
     */
    private MBeanServer jmxServer = null;

    /**
     * @return The local reference of the MBean server
     */
    public MBeanServer getJmxServer() {
        return this.jmxServer;
    }

    /**
     * The JMXServiceURLs used to construct the connector servers attached
     * to the MBean server. We may have several connector servers only
     * if several communication protocols are used, as defined by the
     * Carol configuration.
     */
    private HashMap jmxServiceUrls = new HashMap();
    /**
     * @return the JMXServiceURLs to be used to construct the connector
     * servers attached to the MBean server.
     */
    protected Map getJmxServiceUrls() {
        return this.jmxServiceUrls;
    }

    /**
     * Protocol that can be defined by carol configuration
     */
    private static final String JRMP = "jrmp";
    /**
     * Protocol that can be defined by carol configuration
     */
    private static final String IIOP = "iiop";
    /**
     * Protocol that can be defined by carol configuration
     */
    private static final String JEREMIE = "jeremie";

    private boolean useJrmp = false;

    private boolean useIiop = false;

    private boolean useJeremie = false;

    /**
     * Init the Service. Create the MBeanServer. Configuration information is
     * passed through a Context object.
     * @param ctx context containing service initilaisation parameters
     * @exception ServiceException the service could not be initialized
     */
    public void doInit(Context ctx) throws ServiceException {
        logger = Log.getLogger(Log.JONAS_JMX_PREFIX);
        super.initLogger(Log.getLogger(Log.JONAS_MANAGEMENT_PREFIX));

        // Get the current server name
        try {
            jonasName = (String) ctx.lookup("jonas.name");
        } catch (NamingException ne) {
            logger.log(BasicLevel.DEBUG, "Cannot initialize JMX service " + ne);
            throw new ServiceException("Cannot initialize JMX service", ne);
        }

        // Get the current domain name
        try {
            domainName = (String) ctx.lookup(JProp.DOMAIN_NAME);
        } catch (NamingException ne) {
            // This can't happen as JProp set it in global environment
            // and the ServiceManager passed it to this service
        }

        // Create the MBeanServer
        // ----------------------
        jmxServer = MBeanServerFactory.createMBeanServer(domainName);

        // TEST JMX Remote
        // Get host name
        String hostName = null;
        try {
            hostName = (String) ctx.lookup("host.name");
        } catch (NamingException ne) {
            hostName = "localhost";
        }
        // Get port number
        String portNumber = null;
        int port;
        try {
            portNumber = (String) ctx.lookup("jmxconnector.port");
            port = Integer.valueOf(portNumber).intValue();
        } catch (NamingException ne) {
            port = 0;
        }

        // Determine protocols used by Carol
        String s = CarolConfiguration.getProtocols();
        StringTokenizer st = new StringTokenizer(s, ",", false);
        while (st.hasMoreElements()) {
            String protocol = st.nextToken().trim();
            if (protocol.equals(JRMP)) {
                if (!useJrmp) {
                    useJrmp = true;
                }
            } else if (protocol.equals(IIOP)) {
                if (!useIiop) {
                    useIiop = true;
                }
            } else if (protocol.equals(JEREMIE)) {
                if (!useJeremie) {
                    useJeremie = true;
                }
            }
        }

        String theProtocol = null; // the protocol used in constructed URL
        String theName = null; // the name used in constructed URL
        try {
            if (useJrmp) {
                theProtocol = "rmi";
                theName = "/jndi/jrmpconnector_" + jonasName;
                jmxServiceUrls.put("jrmp", new JMXServiceURL(theProtocol, hostName, port, theName));
                if (port != 0) {
                    // use next port if several protocols and if port numbers are not generated dynamically
                    port++;
                }
            }
            if (useIiop) {
                theProtocol = "iiop";
                theName = "/jndi/iiopconnector_" + jonasName;
                jmxServiceUrls.put("iiop", new JMXServiceURL(theProtocol, hostName, port, theName));
                if (port != 0) {
                    // use next port if several protocols and if port numbers are not generated dynamically
                    port++;
                }
            }
            if (useJeremie) {
                theProtocol = "rmi";
                theName = "/jndi/jeremieconnector_" + jonasName;
                jmxServiceUrls.put("jeremie", new JMXServiceURL(theProtocol, hostName, port, theName));
                if (port != 0) {
                    // use next port if several protocols and if port numbers are not generated dynamically
                    port++;
                }
            }
        } catch (MalformedURLException e) {
            throw new ServiceException("Cannot initialize JMX service", e);
        }

        logger.log(BasicLevel.DEBUG, "JMX Service initialized");
    }

    /**
     * Start the Service Initialization of the service is already done.
     * @exception ServiceException the service could not be started
     */
    public abstract void doStart() throws ServiceException;

    /**
     * Stop this service
     * @exception ServiceException the service could not be stoped
     */
    public abstract void doStop() throws ServiceException;

    public abstract JMXServiceURL[] getConnectorServerURLs();

    /**
     * @return The name of the JOnAS server
     */
    public String getServerName() {
        return this.jonasName;
    }

    /**
     * Remove internal references to the MBeanServer.
     */
    protected void releaseJmxServer() {
        try {
            MBeanServerFactory.releaseMBeanServer(jmxServer);
        } catch (java.lang.IllegalArgumentException ex) {
            // The mbeanServer is not found.
            logger.log(BasicLevel.ERROR, "Cannot find the MBeanServer" + ex);
        }
    }

}