/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JmxServiceImpl.java,v 1.18 2004/07/01 14:57:26 sauthieg Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.jmx;

import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import javax.management.JMException;
import javax.management.ObjectName;
import javax.management.remote.JMXConnectorServer;
import javax.management.remote.JMXConnectorServerFactory;
import javax.management.remote.JMXServiceURL;
import javax.naming.Context;
import javax.naming.InitialContext;

import mx4j.log.CommonsLogger;

import org.objectweb.carol.util.configuration.CarolCurrentConfiguration;

import org.objectweb.jonas.service.JonasAlreadyStartedException;
import org.objectweb.jonas.service.ServiceException;

import org.objectweb.util.monolog.api.BasicLevel;

/**
 * JMX Service implementation.
 * Provides specific doStart() and doStop () methods.

 * @author Adriana Danes.
 *
 */

public class JmxServiceImpl extends AbsJmxServiceImpl {

    /**
     * The name of the JOnAS RMI connector allowing to access the
     * MBeanServer remotelly. Use of this connector was needed before
     * the definition of the Connectors in JMX (before JMX 1.2)
     */
    private String rmiConnectorName = null;

    /**
     * @return The JNDI name of the JOnAS RMI connector
     */
    public String getRmiConnectorName() {
        return this.rmiConnectorName;
    }

    /**
     * Connector servers attached to the MBean server.
     * We may have several connector servers only
     * if several communication protocols are used, as defined by the
     * carol configuration.
     */
    private JMXConnectorServer[] connectorServers = null;

    /**
     * The JMXServiceURLs the connector servers are actually listening on.
     */
    private JMXServiceURL[] connectorServerURLs = null;

    /**
     * Init the logger and then use super method.
     * @param ctx configuration for the init method
     * @throws ServiceException if initialization failed
     */
    public void doInit(Context ctx) throws ServiceException {
        // Set the Mx4Logger that redirects log calls to Jakarta Commons Logging.
        mx4j.log.Log.redirectTo(new CommonsLogger());
        super.doInit(ctx);
    }
    /**
     * Start the Service.
     * Only need to create a RMI connector
     * @exception ServiceException the service could not be started
     */
    public void doStart() throws ServiceException {

        try {
            // Creat a RMI Connector for the JMX agent using the JOnAS RMIConnector class
            // --------------------------------------------------------------------------
            RMIConnector rmiConnector = new RMIConnectorImpl(getJmxServer());
            // Register the connector in JNDI
            InitialContext ictx = new InitialContext();
            rmiConnectorName = "RMIConnector_" + getServerName();
            ictx.bind(rmiConnectorName, rmiConnector);
            ictx.close();

            // Create one or more connector servers
            // (cf. JSR 160, JMX Remote 1.0)
            // ------------------------------------
            int nbConnServers = getJmxServiceUrls().size();
            connectorServers = new JMXConnectorServer[nbConnServers];
            connectorServerURLs = new JMXServiceURL[nbConnServers];
            Map jmxServiceUrls = getJmxServiceUrls();
            Set protocolsSet = jmxServiceUrls.keySet();
            // The environment map, null in case only one connector
            // to be created(use defaults)
            Map environment = null;
            // protocol used with a given connector
            String carolProtocol = null;
            if (nbConnServers == 1) {
                carolProtocol = (String) protocolsSet.toArray()[0];
                JMXServiceURL jmxServiceUrl = (JMXServiceURL) jmxServiceUrls.get(carolProtocol);
                carolProtocol = null; // do not need to know the used protocol in case JOnAS uses only one protocol
                connectorServers[0] = startConnector(jmxServiceUrl, carolProtocol, environment, 0);
            } else {
                // if several connector server, we have to construct an environment for
                // each of them which contains the apropriate properties corresponding to
                // the used protocol, allowing thus to determine the
                // JNDI naming context in which to register the connector
                CarolCurrentConfiguration carolConfig = CarolCurrentConfiguration.getCurrent();
                int i = 0;
                for (Iterator it = protocolsSet.iterator(); it.hasNext();) {
                    carolProtocol = (String) it.next();
                    JMXServiceURL connUrl = (JMXServiceURL) jmxServiceUrls.get(carolProtocol);
                    Properties protocolProps = carolConfig.getRMIProperties(carolProtocol);
                    environment = new HashMap();
                    if (getLogger().isLoggable(BasicLevel.DEBUG)) {
                        getLogger().log(BasicLevel.DEBUG, "Environment used to create and start connector server with URL: " + connUrl + ", contains:");
                    }
                    for (Enumeration pNames = protocolProps.propertyNames(); pNames.hasMoreElements();) {
                        String pName = (String) pNames.nextElement();
                        String pValue = protocolProps.getProperty(pName);
                        if ((pName != null) && (pValue != null)) {
                            environment.put(pName, pValue);
                            if (getLogger().isLoggable(BasicLevel.DEBUG)) {
                                getLogger().log(BasicLevel.DEBUG, "- " + pName + " = " + pValue);
                            }
                        }
                    }
                    environment.put(javax.naming.Context.URL_PKG_PREFIXES, "org.objectweb.jonas.naming");
                    if (getLogger().isLoggable(BasicLevel.DEBUG)) {
                        getLogger().log(BasicLevel.DEBUG, "- " + javax.naming.Context.URL_PKG_PREFIXES + " = " + "org.objectweb.jonas.naming");
                    }
                    connectorServers[i] = startConnector(connUrl, carolProtocol, environment, i++);
                }
            }
        } catch (javax.naming.NameAlreadyBoundException ne) {
            getLogger().log(BasicLevel.DEBUG, "Cannot start JMX service " + ne);
            throw new JonasAlreadyStartedException();
        } catch (javax.naming.ServiceUnavailableException sue) {
            // RMI port used
            String url = null;
            try {
                Context c = new InitialContext();
                Hashtable env = c.getEnvironment();
                url = (String) env.get(InitialContext.PROVIDER_URL);
            } catch (Exception e) {
                // nothing to do
            }
            String err = "RMI Port already in use (url : '" + url + "'). Check your carol.properties !";
            getLogger().log(BasicLevel.DEBUG, err);
            throw new ServiceException(err, sue);
        } catch (Exception e) {
            getLogger().log(BasicLevel.DEBUG, "Cannot start JMX service " + e);
            throw new ServiceException("Cannot start JMX service", e);
        }
    }
    /**
     * Stop this service
     */
    public void doStop() {
        try {
            InitialContext ictx = new InitialContext();
            ictx.unbind("RMIConnector_" + getServerName());
            ictx.close();

            // Deactivates the connector server, that is, stops listening for client connections.
            // Calling this method will also close all client connections that were made by this server.
            // Being a potentialy slow operation, we keep it commented for the moment
            for (int i = 0; i < connectorServers.length; i++) {
                connectorServers[i].stop();
            }
        } catch (Exception e) {
            getLogger().log(BasicLevel.ERROR, "Cannot Unbind Jmx RMI Connector" + e);
        }

        // Remove internal references to the MBeanServer.
        releaseJmxServer();

        getLogger().log(BasicLevel.DEBUG, "JMX Service stopped");
    }

    /**
     * Create and starts a connector server
     * @param serviceURL the address of the new connector server
     * @param environment a set of attributes to determine how the connection is made
     * @param i index to store the actual address of the new connector server
     * @param carolProtocol protocol used by this connector
     * @return the JMXConnectorServer reference
     * @throws IOException communication problem
     * @throws JMException JMX exception
     */
    private JMXConnectorServer startConnector(JMXServiceURL serviceURL, String carolProtocol, java.util.Map environment, int i)
    throws IOException, JMException {
        JMXConnectorServer connectorServer = JMXConnectorServerFactory.newJMXConnectorServer(serviceURL, environment, null);
        // Register the JMXConnectorServer in the MBeanServer
        String protocol = serviceURL.getProtocol();
        String connectorObjectName = null;
        if (carolProtocol == null) {
            connectorObjectName = "connector_" + protocol;
        } else {
            if (carolProtocol.equals("iiop")) {
                connectorObjectName = "connector_" + protocol;
            } else {
                connectorObjectName = "connector_" + protocol + "_" + carolProtocol;
            }
        }
        ObjectName connectorServerName = JonasObjectName.jmxConnectorServer(protocol, connectorObjectName);
        getJmxServer().registerMBean(connectorServer, connectorServerName);
        // Start the JMXConnectorServer
        try {
            connectorServer.start();
            connectorServerURLs[i] = connectorServer.getAddress();
            getLogger().log(BasicLevel.DEBUG, "Connector server " + i + " started on address: " + connectorServer.getAddress());
        } catch (IllegalArgumentException e) {
            throw e;
        }
        return connectorServer;
    }

    /**
     *
     * @return The actual adresses on which listen the created connector servers
     */
    public JMXServiceURL[] getConnectorServerURLs() {
        return this.connectorServerURLs;
    }
}
