/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: LoaderManager.java,v 1.11 2004/07/01 14:51:32 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.server;


import java.io.File;
import java.util.Hashtable;
import java.util.Map;
import java.util.StringTokenizer;

import org.objectweb.jonas.common.JProp;

/**
 * This class create all the ClassLoader necessary for JOnAS.
 *
 * @author Guillaume Sauthier (initial developer)
 */
public class LoaderManager {

    /**
     * Singleton instance of LoaderManager
     */
    private static LoaderManager instance = null;

    /**
     * ClassLoader name for Commons libs
     */
    private static final String COMMONS = "loader.commons";

    /**
     * ClassLoader name for Tools libs
     */
    private static final String TOOLS   = "loader.tools";

    /**
     * ClassLoader name for Applications
     */
    private static final String APPS    = "loader.apps";

    /**
     * ClassLoader name for Catalina
     */
    private static final String CATALINA = "loader.catalina";

    /**
     * JProp instance
     */
    private JProp jp = null;

    /**
     * ClassLoaders
     */
    private Map loaders = null;

    /**
     * $JONAS_ROOT/lib
     */
    private static File jonasLib = null;

    /**
     * $JONAS_ROOT/lib/common
     */
    private static File commonLib = null;

    /**
     * $JONAS_BASE
     */
    private static String jonasBase = null;

    /**
     * $CATALINA_HOME
     */
    private static String catalinaHome = null;

    /**
     * $JETTY_HOME
     */
    private static String jettyHome = null;

    /**
     * true if JOnAS is launched with web container
     */
    private boolean hasWeb = false;

    /**
     * Use the default Classpath ?
     */
    private boolean useDefaultClassPath = false;

    /**
     * is the web container provided by Catalina
     */
    private boolean isCatalina = false;

    /**
     * Create a LoaderManager instance
     */
    private LoaderManager() {
        // Get the $JONAS_ROOT/lib directories
        String jonasRoot = System.getProperty("install.root");
        if (jonasRoot == null) {
            System.err.println("JONAS_ROOT is not set");
            System.exit(2);
        }

        jonasLib = new File(jonasRoot, "lib");
        commonLib = new File(jonasLib, "common");

        // Init loaders map
        loaders = new Hashtable();
    }

    /**
     * @return Returns the unique instance of LoaderManager.
     */
    public static LoaderManager getInstance() {
        if (instance == null) {
            instance = new LoaderManager();
        }
        return instance;
    }

    /**
     * Initialize the LoaderManager with a JProp instance.
     *
     * @param props JProp used to configure JOnAS
     */
    public void init(JProp props) {
        jp = props;
        // Get $JONAS_BASE
        jonasBase = JProp.getJonasBase();

        useDefaultClassPath = jp.getValueAsBoolean("jonas.default.classloader", false);

        // find web services
        String[] services = props.getValueAsArray("jonas.services");
        if (services != null) {
            for (int i = 0; i < services.length; i++) {
                if (services[i].equals("web")) {
                    hasWeb = true;
                    String classname = props.getValue("jonas.service.web.class");
                    if (classname.matches(".*Catalina.*")) {
                        isCatalina = true;
                    }
                }
            }
        }

        catalinaHome = System.getProperty("catalina.home");
        jettyHome = System.getProperty("jetty.home");

        // Set in all case catalina.home and base to $JONAS_BASE
        // if library is found. Useful if a user want to run in
        // a same environment JOnAS/Tomcat and Tomcat with different
        // versions
        File jarCatalina = new File(jonasLib, "web_catalina_commons.jar");
        if (jarCatalina.exists()) {
            System.setProperty("catalina.home", jonasBase);
            System.setProperty("catalina.base", jonasBase);
            catalinaHome = jonasBase;
        }

        // Set jetty.home to $JONAS_BASE if library is found
        // and if jetty.home isn't set.
        if (jettyHome == null) {
            File jarJetty = new File(jonasLib, "web_jetty.jar");
            if (jarJetty.exists()) {
                System.setProperty("jetty.home", jonasBase);
                jettyHome = jonasBase;
            }
        }

    }

    /**
     * @return Returns the Commons ClassLoader
     * @throws Exception When ClassLoader cannot be created.
     */
    public JClassLoader getCommonsLoader() throws Exception {
        JClassLoader commons = getLoader(COMMONS);
        if (commons == null) {
            commons = createCommonsClassLoader();
            setLoader(COMMONS, commons);
        }
        return commons;
    }

    /**
     * @return returns the Tools ClassLoader
     * @throws Exception When ClassLoader cannot be created
     */
    public JClassLoader getToolsLoader() throws Exception {
        JClassLoader tools = getLoader(TOOLS);
        if (tools == null) {
            tools = createToolsClassLoader();
            setLoader(TOOLS, tools);
        }
        return tools;
    }

    /**
     * @return Returns the Application ClassLoader
     * @throws Exception When ClassLoader cannot be created
     */
    public JClassLoader getAppsLoader() throws Exception {
        JClassLoader apps = getLoader(APPS);
        if (apps == null) {
            apps = createAppsClassLoader();
            setLoader(APPS, apps);
        }
        return apps;
    }

    /**
     * @return Returns the Catalina ClassLoader
     * @throws Exception When ClassLoader cannot be created.
     */
    public JClassLoader getCatalinaLoader() throws Exception {
        JClassLoader catalina = getLoader(CATALINA);
        if (catalina == null) {
            catalina = createCatalinaClassLoader();
            setLoader(CATALINA, catalina);
        }
        return catalina;
    }

    /**
     * Return an array containing all the resources declared in the classpath
     * environment variable.
     *
     * @param xtracp XTRA classpath
     *
     * @return a list containing all the resources declared in the classpath
     * environment variable.
     *
     * @throws Exception if a resource cannot be found.
     */
    private static JURLs getClassPath(boolean xtracp) throws Exception {
        String cpprop = "java.class.path";
        if (xtracp) {
            cpprop = "jonas.classpath";
        }
        String cpValue = System.getProperty(cpprop);
        //System.out.println("classpath=" + cpValue); // DEBUG
        String sep = File.pathSeparator;
        JURLs jurls = new JURLs();
        StringTokenizer tokenizer = new StringTokenizer(cpValue, sep);
        while (tokenizer.hasMoreTokens()) {
            File file = new File(tokenizer.nextToken());
            if (file.isDirectory()) {
                jurls.addDir(file);
            } else {
                // do not readd bootstrap.jar inside classloader
                if (!file.getName().equals("ow_jonas_bootstrap.jar")) {
                    jurls.add(file);
                }
            }
        }
        return jurls;
    }

    /**
     * Returns a JClassLoader.
     *
     * @param name name of the loader
     *
     * @return Returns the JClassLoader
     */
    private JClassLoader getLoader(String name) {
        return (JClassLoader) loaders.get(name);
    }

    /**
     * Add a couple (name, ClassLoader).
     *
     * @param name ClassLoader name
     * @param loader the ClassLoader
     */
    private void setLoader(String name, JClassLoader loader) {
        loaders.put(name, loader);
    }

    /**
     * Constructs the Commons ClassLoader.
     *
     * @return Returns the Commons ClassLoader.
     * @throws Exception When ClassLoader cannot be created from CLASSPATH or resources cannot be added to URLs List.
     */
    private JClassLoader createCommonsClassLoader() throws Exception {

        JURLs jurls = new JURLs();

        // add classpath in loader
        jurls.merge(getClassPath(useDefaultClassPath));

        // Load now mail.jar because it must be loaded before activation.jar
        jurls.add(new File(commonLib, "j2ee" + File.separator + "mail"), ".jar");

        // activation.jar needed at least for GenIC
        jurls.add(new File(commonLib, "j2ee" + File.separator + "activation"), ".jar");

        jurls.merge(getJonasResources());

        // Add the specific resource of the web_container (tomcat or jetty)
        // service
        if (hasWeb) {
            // resources for catalina have been put in apps ClassLoader

            // add resources for jetty if required
            if (jettyHome != null && !isCatalina) {
                jurls.merge(getJettyResources(new File(jettyHome)));
            }
            // else user has not set the webcontainer HOME var
        }

        // add resources for WebServices
        jurls.merge(getAxisResources());

        // create a parent classloader with classpath within
        JClassLoader classpathLoader = null;
        // if useDefaultClassPath set, we add the jonas.classpath value inside parent ClasLoader
        classpathLoader = new JClassLoader(getClassPath(useDefaultClassPath).toURLs());

        return new JClassLoader(jurls.toURLs(), classpathLoader);
    }

    /**
     * Constructs the Tools ClassLoader using Commons ClassLoader as parent.
     *
     * @return Returns the Tools ClassLoader
     *
     * @throws Exception When resources cannot be added to URL list.
     */
    private JClassLoader createToolsClassLoader() throws Exception {
        JURLs jurls = new JURLs();

        // add tools lib
        jurls.add(new File(commonLib, "velocity"), ".jar");
        jurls.add(new File(commonLib, "jakarta" + File.separator + "commons-digester"), ".jar");

        // monolog velocity wrapper
        jurls.add(new File(commonLib, "log" + File.separator + "ow_util_log_wrp_velo.jar"));

        jurls.add(new File(commonLib, "ow_jonas_genic.jar"));
        jurls.add(new File(commonLib, "ow_jonas_newbean.jar"));
        jurls.add(new File(commonLib, "ow_jonas_wsgen.jar"));
        jurls.add(new File(commonLib, "ow_jonas_xml.jar"));

        jurls.add(new File(commonLib, "carol" + File.separator + "ow_carol_cmic.jar"));

        jurls.add(new File(commonLib, "jorm" + File.separator + "ow_jorm_compiler.jar"));

        return new JClassLoader(jurls.toURLs(), getCommonsLoader());
    }

    /**
     * Constructs the Application ClassLoader with Commons ClassLoader as parent.
     * At this time, this is an empty ClassLoader
     *
     * @return the Application ClassLoader
     *
     * @throws Exception When resources cannot be added to URL list.
     */
    private JClassLoader createAppsClassLoader() throws Exception {
        JURLs jurls = new JURLs();

        if (hasWeb) {
            // add resources for catalina if required
            if (catalinaHome != null && isCatalina) {
                jurls.merge(getCatalinaResources(new File(catalinaHome)));
            }
        }
        return new JClassLoader(jurls.toURLs(), getCommonsLoader());
    }

    /**
     * Constructs the Catalina ClassLoader with Applications ClassLoader as parent.
     *
     * @return the Catalina ClassLoader
     *
     * @throws Exception When resources cannot be added to URL list.
     */
    private JClassLoader createCatalinaClassLoader() throws Exception {
        JURLs jurls = new JURLs();
        File jarf = new File(jonasLib, "web_catalina_server.jar");

        if (jarf.exists()) {
            jurls.add(jarf);
        } else {
            if (new File(commonLib, "ow_jonas_tomcat4.jar").exists()) {
                jurls.add(new File(commonLib, "ow_jonas_tomcat4.jar"));
            } else if (new File(commonLib, "ow_jonas_tomcat5.jar").exists()) {
                jurls.add(new File(commonLib, "ow_jonas_tomcat5.jar"));
            } else {
                System.out.println("WARN : No ow_jonas_tomcat*.jar found in lib/common directory.");
            }
            jurls.add(new File(catalinaHome, "server" + File.separator + "lib"));
        }

        return new JClassLoader(jurls.toURLs(), getAppsLoader());
    }

    /**
     * Return an array containing all the resources used for the integration of
     * Catalina.
     *
     * @param catalinaHomeDir the home directory of Catalina.
     *
     * @return a list containing all the resources used for the integration of
     *         Catalina.
     *
     * @throws Exception if a resource cannot be found.
     */
    private static JURLs getCatalinaResources(File catalinaHomeDir) throws Exception {

        JURLs jurls = new JURLs();
        File jarf = new File(jonasLib, "web_catalina_commons.jar");

        if (jarf.exists()) {
            jurls.add(jarf);
        } else {
            jurls.add(new File(catalinaHomeDir, "bin" + File.separator + "bootstrap.jar"));
            jurls.add(new File(catalinaHomeDir, "common" + File.separator + "lib"));
        }
        return jurls;
    }

    /**
     * Return an array containing all the resources used for the integration of
     * Axis.
     *
     * @return a list containing all the resources used for the integration of
     *         Axis.
     *
     * @throws Exception if a resource cannot be found.
     */
    private static JURLs getAxisResources() throws Exception {
        JURLs jurls = new JURLs();

        File jarf = new File(jonasLib, "webservices_axis.jar");

        // if ant install done
        if (jarf.exists()) {
            jurls.add(jarf);
        } else {
            // add manually all axis specific files
            jurls.add(new File(commonLib, "ws" + File.separator + "axis"), ".jar");

            if (new File(commonLib, "ow_jonas_axis.jar").exists()) {
                jurls.add(new File(commonLib, "ow_jonas_axis.jar"));
            } else {
                System.out.println("WARN : No ow_jonas_axis.jar found in lib/common directory.");
            }
        }
        return jurls;
    }

    /**
     * Return an array containing all the resources used for WebServices.
     *
     * @return a list containing all the resources used for WebServices.
     *
     * @throws Exception if a resource cannot be found.
     */
    private static JURLs getWebServicesResources() throws Exception {
        JURLs jurls = new JURLs();

        jurls.add(new File(commonLib, "ws" + File.separator + "wsdl4j"), ".jar");
        jurls.add(new File(commonLib, "ws" + File.separator + "namespace"), ".jar");
        // add JAX-R Resources
        jurls.add(new File(commonLib, "ws" + File.separator + "jaxr"), ".jar");

        return jurls;
    }

    /**
     * Return an array containing all the resources used for the integration of
     * Jetty.
     *
     * @param jettyHomeDir the home directory of Jetty.
     *
     * @return a list containing all the resources used for the integration of
     *         Jetty.
     *
     * @throws Exception if a resource cannot be found.
     */
    private static JURLs getJettyResources(File jettyHomeDir) throws Exception {

        JURLs jurls = new JURLs();

        File jarf = new File(jonasLib, "web_jetty.jar");

        if (jarf.exists()) {
            jurls.add(jarf);
        } else {
            String jettyLib = jettyHomeDir + File.separator + "lib";
            jurls.add(new File(jettyLib, "org.mortbay.jetty.jar"));

            String jettyExt = jettyHomeDir + File.separator + "ext";
            jurls.add(new File(jettyExt, "ant.jar"));
            jurls.add(new File(jettyExt, "jasper-compiler.jar"));
            jurls.add(new File(jettyExt, "jasper-runtime.jar"));
            jurls.add(new File(jettyExt, "commons-el.jar"));
            jurls.add(new File(jettyLib, "org.mortbay.jmx.jar"));
            jurls.add(new File(jettyLib, "javax.servlet.jar"));
            jurls.add(new File(commonLib, "ow_jonas_jetty.jar"));
        }
        return jurls;
    }

    /**
     * Return an array containing all the resources used for JOnAS
     *
     * @return a list containing all the resources.
     *
     * @throws Exception if a resource cannot be found.
     */
    private static JURLs getJonasResources() throws Exception {

        JURLs jurls = new JURLs();

        File jarf = new File(jonasLib, "jonas.jar");

        if (jarf.exists()) {
            jurls.add(jarf);
        } else {
            // add Orb Independant JOnAS resources
            jurls.add(new File(commonLib, "ow_jonas.jar"));
            jurls.add(new File(commonLib, "ow_jonas_jdbc_ra.jar"));

            // add external jars needed for JOnAS
            jurls.add(new File(commonLib, "j2ee"), ".jar");
            //jurls.add(new File(commonLib, "acme"), ".jar");
            jurls.add(new File(commonLib, "joram"), ".jar");

            jurls.add(new File(commonLib, "jdo"), ".jar");

            // jorm
            jurls.add(new File(commonLib, "jorm" + File.separator + "ow_jorm_api.jar"));
            jurls.add(new File(commonLib, "jorm" + File.separator + "ow_jorm_core.jar"));
            jurls.add(new File(commonLib, "jorm" + File.separator + "ow_jorm_runtime.jar"));
            jurls.add(new File(commonLib, "jorm" + File.separator + "ow_perseus_connection_holder.jar"));
            jurls.add(new File(commonLib, "jorm" + File.separator + "ow_perseus_cache.jar"));

            jurls.add(new File(commonLib, "jotm"), ".jar");
            jurls.add(new File(commonLib, "jmx"), ".jar");

            // monolog
            jurls.add(new File(commonLib, "log" + File.separator + "ow_util_log_core.jar"));
            jurls.add(new File(commonLib, "log" + File.separator + "ow_util_log_wrp_common.jar"));
            jurls.add(new File(commonLib, "log" + File.separator + "ow_util_log_wrp_log4j.jar"));
            jurls.add(new File(commonLib, "log" + File.separator + "ow_util_log_wrp_pw.jar"));
            jurls.add(new File(commonLib, "log" + File.separator + "ow_util_log_api.jar"));
            jurls.add(new File(commonLib, "log" + File.separator + "ow_util_log_file.jar"));
            jurls.add(new File(commonLib, "log" + File.separator + "ow_util_log_wrp_javalog.jar"));
            jurls.add(new File(commonLib, "log" + File.separator + "ow_util_log_wrp_p6spy.jar"));

            jurls.add(new File(commonLib, "medor"), ".jar");
            jurls.add(new File(commonLib, "carol" + File.separator + "ow_carol_core.jar"));
            jurls.add(new File(commonLib, "carol" + File.separator + "javagroups"), ".jar");
            jurls.add(new File(commonLib, "jakarta" + File.separator + "commons-beanutils"), ".jar");
            jurls.add(new File(commonLib, "jakarta" + File.separator + "commons-collections"), ".jar");
            jurls.add(new File(commonLib, "jakarta" + File.separator + "commons-logging"), ".jar");
            jurls.add(new File(commonLib, "jakarta" + File.separator + "commons-modeler"), ".jar");

            jurls.add(new File(commonLib, "jonathan"), ".jar");
            jurls.add(new File(commonLib, "p6spy"), ".jar");

            // speedo
            jurls.add(new File(commonLib, "speedo" + File.separator + "speedo_for_jonas_ra.jar"));

            //Hsql
            jurls.add(new File(commonLib, "hsqldb"), ".jar");
            jurls.merge(getWebServicesResources());
        }

        // add jonas.base/lib/ext only if jonas.base != jonas.root
        // jonas.base/lib/ext is added before jonas.root/lib/ext
        if (!System.getProperty("install.root").toLowerCase().equals(jonasBase.toLowerCase())) {
            File jonasBaseLibExt = new File(jonasBase + File.separator + "lib" + File.separator + "ext");
            if (jonasBaseLibExt.exists()) {
                jurls.add(jonasBaseLibExt, ".jar");
            }
        }

        // add jonas.root/lib/ext
        jurls.add(new File(jonasLib, "ext"), ".jar");

        return jurls;
    }

}
