/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial Developer : Delplanque Xavier & Sauthier Guillaume
 * --------------------------------------------------------------------------
 * $Id: FileWSDLHandler.java,v 1.11 2004/05/10 13:01:12 sauthieg Exp $
 * --------------------------------------------------------------------------
*/

package org.objectweb.jonas.ws.handler;

import org.objectweb.jonas.ws.WSServiceException;

import org.objectweb.jonas_ws.deployment.api.WSDLFile;

import java.io.File;
import java.io.Writer;
import java.io.OutputStreamWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.charset.IllegalCharsetNameException;
import java.nio.charset.UnsupportedCharsetException;

import java.util.Properties;

import javax.wsdl.WSDLException;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLWriter;


/**
 * In charge of published a given WSDL in a directory.
 * properties required :<br/>
 * - <code>jonas.service.publish.file.directory</code> : directory name where WSDLs will be placed
 *
 * @author Xavier Delplanque
 */
public class FileWSDLHandler implements WSDLHandler {

    /** WSDL Directory */
    private File location;

    /** charset */
    private Charset cs;

    /** directory property name */
    private static final String OUTPUT_DIRECTORY = "jonas.service.publish.file.directory";

    /** encoding mode */
    private static final String ENCODING = "jonas.service.publish.file.encoding";

    /**
     * Return a new instance of FileWSDLHandler.
     *
     * @param props Properties used to configure FileHandler
     *
     * @throws WSServiceException When Cannot create the directory
     *         (if it don't exist or not set in property file).
     */
    public FileWSDLHandler(Properties props) throws WSServiceException {

        String directory = props.getProperty(OUTPUT_DIRECTORY);
        String encoding = props.getProperty(ENCODING, "UTF-8");

        try {
            location = new File(directory).getCanonicalFile();

            if (!location.exists()) {
                // if the given file doesn't exist, create it.
                location.mkdirs();
            }

            cs = Charset.forName(encoding);

        } catch (IOException ioe) {
            throw new WSServiceException(
                "cannot find/create the publishing directory '" + directory + "'",
                ioe);
        } catch (IllegalCharsetNameException icsne) {
            throw new WSServiceException(
                "Illegal Charset '" + encoding + "'",
                icsne);
        } catch (UnsupportedCharsetException ucse) {
            throw new WSServiceException(
                "Charset '" + encoding + "' not supported on this platform.",
                ucse);
        }
    }

    /**
     * Publish the given WSDL to the directory location.
     *
     * @param wsdl the wsdl file to publish.
     *
     * @throws WSServiceException When publication fails.
     */
    public void publish(WSDLFile wsdl) throws WSServiceException {
        // build the fully qualified file name for the published wsdl file
        String filePath = wsdl.getName();
        int ind = filePath.lastIndexOf("/");
        String fileName;

        if (ind != -1) {
            fileName = filePath.substring(ind);
        } else {
            fileName = filePath;
        }

        try {
            // write the wsdl file in the directory

            Writer fwriter = new OutputStreamWriter(new FileOutputStream(new File(location, fileName)), cs);

            WSDLFactory factory = WSDLFactory.newInstance();
            WSDLWriter writer = factory.newWSDLWriter();
            writer.writeWSDL(wsdl.getDefinition(), fwriter);
            fwriter.flush();
            fwriter.close();
        } catch (IOException ioe) {
            throw new WSServiceException("Error with writer of file '"
                                         + fileName + "'", ioe);
        } catch (WSDLException we) {
            throw new WSServiceException("Error with wsdl file '" + filePath
                                         + "' publishing in directory '" + location
                                         + "'", we);
        }
    }
}
