/**
 * JOnAS : Java(TM) OpenSource Application Server
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial Developer : Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: AxisWsClientGenerator.java,v 1.7 2004/07/06 12:59:11 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ws.wsgen.generator.axis;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.apache.velocity.VelocityContext;

import org.objectweb.common.Cmd;

import org.objectweb.jonas_lib.I18n;

import org.objectweb.jonas_ws.deployment.api.MappingFile;
import org.objectweb.jonas_ws.deployment.api.ServiceRefDesc;
import org.objectweb.jonas_ws.wsgen.WsGenException;
import org.objectweb.jonas_ws.wsgen.archive.Archive;
import org.objectweb.jonas_ws.wsgen.archive.EjbJar;
import org.objectweb.jonas_ws.wsgen.archive.J2EEArchive;
import org.objectweb.jonas_ws.wsgen.archive.WebApp;
import org.objectweb.jonas_ws.wsgen.ddmodifier.WsClientDDModifier;
import org.objectweb.jonas_ws.wsgen.generator.Config;
import org.objectweb.jonas_ws.wsgen.generator.WsClientGenerator;
import org.objectweb.jonas_ws.wsgen.utils.XMLUtils;

import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Generate WebServices client files dedicated to axis. <ul><li>
 * client-config.wsdd : if needed</li><li>java sources : from WSDL</li>
 * </ul>
 * @author Guillaume sauthier
 */
public class AxisWsClientGenerator extends WsClientGenerator {

    /** unique JVelocity instance */
    private static JVelocity jvelocity = null;

    /**
     * Axis WSDL to Java main class
     */
    private static final String WSDL2JAVA = "org.apache.axis.wsdl.WSDL2Java";

    /**
     * jonas-init-param name for client configuration file declaration
     */
    private static final String CLIENT_CONFIG = "axis.clientConfigFile";

    /** count generated files to assure names unicity */
    private static int count = 0;

    /** generated config file */
    private File generated = null;

    /**
     * WEB-INF/ prefix
     */
    private static final String WEB_PREFIX = "WEB-INF/";

    /**
     * I18n
     */
    private static I18n i18n = I18n.getInstance(AxisWsClientGenerator.class);

    /**
     * Creates a new AxisWsClientGenerator
     * @param config Generator Configuration
     * @param srd WebService Endpoint description
     * @param ddm Web DD Modifier
     * @param archive client archive containing WSDL
     * @throws WsGenException When instanciation fails
     */
    public AxisWsClientGenerator(Config config, ServiceRefDesc srd, WsClientDDModifier ddm, Archive archive)
            throws WsGenException {
        super(config, srd, ddm, archive);

        // init velocity
        if (jvelocity == null) {
            jvelocity = new JVelocity("deploy_client.vm");
        }
    }

    /**
     * generate axis specific files
     * @throws WsGenException if WSDL cannot be found in archive
     */
    public void generate() throws WsGenException {

        // the source generation is possible only when a
        // WSDL Definition is provided
        if ((getRef().getWsdlFileName() != null)
                && !getRef().getServiceInterface().getName().equals("javax.xml.rpc.Service")) {
            // Source generation for the client
            String cmdJava = getConfig().getJavaHomeBin() + "java";
            String wsdl = null;

            // Command preparation
            Cmd axisCmd = new Cmd(cmdJava);
            axisCmd.addArgument("-classpath");
            axisCmd.addArgument(getConfig().getClasspath());
            axisCmd.addArgument(WSDL2JAVA);

            // get informations from ServiceRef ...
            // WSDL file
            if (getArchive().isPacked()) {
                // URL Loading
                try {
                    String jarpath = getArchive().getRootFile().getCanonicalFile().toURL().toExternalForm();
                    wsdl = "jar:" + jarpath + "!/" + getRef().getWsdlFileName();

                } catch (IOException ioe) {
                    String err = getI18n().getMessage("AxisWsClientGenerator.generate.wsdl",
                            getRef().getWsdlFileName(), getArchive().getRootFile());
                    throw new WsGenException(err, ioe);
                }
            } else {
                // File searching
                try {
                    wsdl = new File(getArchive().getRootFile(), getRef().getWsdlFileName()).toURL().toExternalForm();

                } catch (IOException ioe) {
                    String err = getI18n().getMessage("AxisWsClientGenerator.generate.wsdl",
                            getRef().getWsdlFileName(), getArchive().getRootFile());
                    throw new WsGenException(err, ioe);
                }
            }

            // Mapping File
            MappingFile mf = getRef().getMappingFile();

            if (mf != null) {
                for (Iterator i = mf.getMappings().keySet().iterator(); i.hasNext();) {
                    String namespace = (String) i.next();
                    String pkg = (String) mf.getMappings().get(namespace);
                    axisCmd.addArgument("--NStoPkg");
                    axisCmd.addArgument(namespace + "=" + pkg);
                }
            }

            // Set the output directory
            axisCmd.addArgument("--output");
            axisCmd.addArgument(getSources().getPath());

            if (getConfig().isVerbose()) {
                axisCmd.addArgument("--verbose");
            }

            if (getConfig().isDebug()) {
                axisCmd.addArgument("--Debug");
            }

            axisCmd.addArgument(wsdl);

            getLogger().log(BasicLevel.DEBUG, "Running '" + axisCmd.toString() + "'");
            if (axisCmd.run()) {
                getLogger().log(BasicLevel.INFO, "Web Services Classes successfully generated by Axis.");
            } else {
                String err = getI18n().getMessage("AxisWsClientGenerator.generate.WSDL2Java");
                throw new WsGenException(err);
            }
        }

        // Client-config.wsdd ??
        // only if handlers are specified in ServiceRef
        // but if noConfig set to true, assume that
        // user configure himself the client. (expert)
        // Could we create configuration ?
        if (hasClientConfigFile(getRef())) {
            // create deploy.wsdd
            // use velocity template
            // build a unique file name
            String filename = "client-config-" + (count++) + ".wsdd";

            VelocityContext vctx = VContextFactory.getContext(getRef());

            getLogger().log(BasicLevel.DEBUG, "Creating '" + filename + "'");
            generated = new File(getSources(), filename);

            jvelocity.generate(generated, vctx);

        }
        // End !noConfig
    }

    /**
     * Returns true if given service-ref need a client configuration files
     * @param ref Service Ref
     * @return true if given service-ref need a client configuration files
     */
    private boolean hasClientConfigFile(ServiceRefDesc ref) {
        // no configuration asked
        if (getConfig().isNoConfig()) {
            return false;
        }

        MappingFile mf = ref.getMappingFile();

        if (mf != null) {
            // mapping file defined
            if (mf.getXmlTypeMappings().hasNext()) {
                // we have 1 mapping at least
                return true;
            } else {
                // no mapping
                // return true if ref has Handlers
                return (ref.getHandlerRefs().size() != 0);
            }
        } else {
            // no mapping file
            // return true if ref has Handlers
            return (ref.getHandlerRefs().size() != 0);
        }
    }

    /**
     * Add generated files in given archive
     * @param archive archive where generated fils will be added.
     * @throws WsGenException when files cannot be added
     */
    public void addFiles(Archive archive) throws WsGenException {
        if (archive instanceof WebApp) {
            archive.addDirectoryIn("WEB-INF/classes/", getClasses());

            if (generated != null) {
                archive.addFileIn("WEB-INF/", generated);

                // ensure the optionnal descriptor exists
                if (!getModifier().hasJonasServiceRef()) {
                    if (!getArchive().getContainedFiles().contains("WEB-INF/jonas-web.xml")) {
                        // jonas-web.xml doesn't exists
                        createEmptyJonasWeb((J2EEArchive) archive);
                    }
                    Element jsr = getModifier().createJonasServiceRef(getRef().getServiceRefName());
                    // update
                    getModifier().setElement(jsr);

                }

                // add init param
                getModifier().addJonasInitParam(CLIENT_CONFIG, WEB_PREFIX + generated.getName());
            }
        } else if (archive instanceof EjbJar) {
            archive.addDirectory(getClasses());

            if (generated != null) {
                archive.addFileIn("META-INF/", generated);

                // ensure the optionnal descriptor exists
                if (!getModifier().hasJonasServiceRef()) {
                    Element jsr = getModifier().createJonasServiceRef(getRef().getServiceRefName());
                    // update
                    getModifier().setElement(jsr);
                }

                // add init param
                getModifier().addJonasInitParam(CLIENT_CONFIG, "META-INF/" + generated.getName());
            }
        } else {
            archive.addDirectory(getClasses());

            if (generated != null) {
                archive.addFileIn("META-INF/", generated);

                // ensure the optionnal descriptor exists
                if (!getModifier().hasJonasServiceRef()) {
                    if (!getArchive().getContainedFiles().contains("META-INF/jonas-client.xml")) {
                        // jonas-client.xml doesn't exists
                        createEmptyJonasClient((J2EEArchive) archive);
                    }
                    Element jsr = getModifier().createJonasServiceRef(getRef().getServiceRefName());
                    // update
                    getModifier().setElement(jsr);

                }

                // add init param
                getModifier().addJonasInitParam(CLIENT_CONFIG, "META-INF/" + generated.getName());
            }
        }
    }

    /**
     * Add an empty jonas-web.xml in given J2EEArchive.
     * @param archive archive to be updated
     */
    private void createEmptyJonasWeb(J2EEArchive archive) {
        Document doc = XMLUtils.newJonasWeb();
        archive.getDescriptors().put("WEB-INF/jonas-web.xml", doc);
        getModifier().setDocument(doc);
    }

    /**
     * Add an empty jonas-client.xml in given J2EEArchive.
     * @param archive archive to be updated
     */
    private void createEmptyJonasClient(J2EEArchive archive) {
        Document doc = XMLUtils.newJonasClient();
        archive.getDescriptors().put("META-INF/jonas-client.xml", doc);
        getModifier().setDocument(doc);
    }

    /**
     * @return Returns the i18n.
     */
    public static I18n getI18n() {
        return i18n;
    }
}